/**********************************************************************
 *<
	FILE: MirrorTVs.h

	DESCRIPTION:	Includes for Plugins

	CREATED BY:

	HISTORY:

 *>	Copyright (c) 2000, All Rights Reserved.
 **********************************************************************/

#ifndef __MIRRORTVS__H
#define __MIRRORTVS__H

#include "Max.h"
#include "resource.h"
#include "istdplug.h"
#include "iparamb2.h"
#include "iparamm2.h"

#include "meshadj.h"
#include "XTCObject.h"


extern TCHAR *GetString(int id);

extern HINSTANCE hInstance;

#define MIRRORTVS_CLASS_ID	Class_ID(0x363ba61, 0x3a1f15c0)

#define PBLOCK_REF	0

class MirrorTVs;

class MirrorTVsDlgProc : public ParamMap2UserDlgProc {
	public:
		MirrorTVs *mod;
		MirrorTVsDlgProc(MirrorTVs *m) {mod = m;}
		BOOL DlgProc(TimeValue t,IParamMap2* map,HWND hWnd,UINT msg,WPARAM wParam,LPARAM lParam);
		void DeleteThis() { delete this;}
	};


class FaceElement
{
public:
int index;
float val;
};

class FaceElementList2
{
public:
Tab<FaceElement> data;
};

class FaceRadixData
{
private:
	Tab<FaceElementList2*> faceElements;
public:
	int numberCells;
	float min;
	float max;
	float divisor;
	FaceElement *GetElementList(float val, int &count);
	BuildRadixData(float threshold, int count, FaceElement *data);
	~FaceRadixData() {FreeFaceElementList();};
	FreeFaceElementList();
};



class MirroredFace
{
public:
	int index;
	int mirrorIndex;
	int order[3];
	Point3 mapA,mapB,mapC;

};

class MirrorTVs : public Modifier {
	public:


		// Parameter block
		IParamBlock2	*pblock;	//ref 0


		static IObjParam *ip;			//Access to the interface
		
		// From Animatable
		TCHAR *GetObjectName() { return GetString(IDS_CLASS_NAME); }

		//From Modifier
		ChannelMask ChannelsUsed()  { return GEOM_CHANNEL|TOPO_CHANNEL|VERTCOLOR_CHANNEL|PART_DISPLAY|TEXMAP_CHANNEL; }
		//TODO: Add the channels that the modifier actually modifies
		ChannelMask ChannelsChanged() { return GEOM_CHANNEL|VERTCOLOR_CHANNEL|PART_DISPLAY|TEXMAP_CHANNEL; }
		//TODO: Return the ClassID of the object that the modifier can modify
		Class_ID InputType() {return triObjectClassID;}

		void ModifyObject(TimeValue t, ModContext &mc, ObjectState *os, INode *node);
		void NotifyInputChanged(Interval changeInt, PartID partID, RefMessage message, ModContext *mc);

		void NotifyPreCollapse(INode *node, IDerivedObject *derObj, int index);
		void NotifyPostCollapse(INode *node,Object *obj, IDerivedObject *derObj, int index);


		Interval LocalValidity(TimeValue t);

		// From BaseObject
		//TODO: Return true if the modifier changes topology
		BOOL ChangeTopology() {return FALSE;}		
		
		CreateMouseCallBack* GetCreateMouseCallBack() {return NULL;}

		BOOL HasUVW();
		void SetGenUVW(BOOL sw);


		void BeginEditParams(IObjParam *ip, ULONG flags,Animatable *prev);
		void EndEditParams(IObjParam *ip, ULONG flags,Animatable *next);


		Interval GetValidity(TimeValue t);

		// Automatic texture support
		
		// Loading/Saving
		IOResult Load(ILoad *iload);
		IOResult Save(ISave *isave);

		//From Animatable
		Class_ID ClassID() {return MIRRORTVS_CLASS_ID;}		
		SClass_ID SuperClassID() { return OSM_CLASS_ID; }
		void GetClassName(TSTR& s) {s = GetString(IDS_CLASS_NAME);}

		RefTargetHandle Clone( RemapDir &remap );
		RefResult NotifyRefChanged(Interval changeInt, RefTargetHandle hTarget, 
			PartID& partID,  RefMessage message);


		int NumSubs() { return 1; }
		TSTR SubAnimName(int i) { return GetString(IDS_PARAMS); }				
		Animatable* SubAnim(int i) { return pblock; }

		// TODO: Maintain the number or references here
		int NumRefs() { return 1; }
		RefTargetHandle GetReference(int i) { return pblock; }
		void SetReference(int i, RefTargetHandle rtarg) { pblock=(IParamBlock2*)rtarg; }




		int	NumParamBlocks() { return 1; }					// return number of ParamBlocks in this instance
		IParamBlock2* GetParamBlock(int i) { return pblock; } // return i'th ParamBlock
		IParamBlock2* GetParamBlockByID(BlockID id) { return (pblock->ID() == id) ? pblock : NULL; } // return id'd ParamBlock

		void DeleteThis() { delete this; }		
		//Constructor/Destructor

		MirrorTVs();
		~MirrorTVs();	
		
		int Display(TimeValue t, INode* inode, ViewExp *vpt, int flagst, ModContext *mc);
		Matrix3 CompMatrix(TimeValue t,INode *inode,ModContext *mc);

		int NumSubObjTypes();
		ISubObjType *GetSubObjType(int i);

		BitArray displayVerts;
		Tab<Point3> mirrorVerts;
		Tab<Point3> strayDisplayVerts;
		Tab<MirroredFace> mirrorFaces;
		BOOL useDrawTM;
		Matrix3 drawTM;

};


class MirrorTVsClassDesc:public ClassDesc2 {
	public:
	int 			IsPublic() { return TRUE; }
	void *			Create(BOOL loading = FALSE) { return new MirrorTVs(); }
	const TCHAR *	ClassName() { return GetString(IDS_CLASS_NAME); }
	SClass_ID		SuperClassID() { return OSM_CLASS_ID; }
	Class_ID		ClassID() { return MIRRORTVS_CLASS_ID; }
	const TCHAR* 	Category() { return GetString(IDS_CATEGORY); }

	const TCHAR*	InternalName() { return _T("MirrorTVs"); }	// returns fixed parsable name (scripter-visible name)
	HINSTANCE		HInstance() { return hInstance; }				// returns owning module handle

};

enum { mirrortvs_params };


//TODO: Add enums for various parameters
enum { 
	pb_mapchannel,pb_axis,pb_offset,pb_threshold,
	pb_usenode,pb_node,
	pb_copymatids,pb_copysmoothinggroups,
	pb_instanceuvs,
	pb_copystrayverts

};

enum { kX,kY,kZ};

class sMyEnumProc : public DependentEnumProc 
	{
      public :
      virtual int proc(ReferenceMaker *rmaker); 
      INodeTab Nodes;              
	};




#endif // __MIRRORTVS__H
